/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 9.6, Version 1.0, 2001-04-24
****************************************************/

USE Northwind
GO

-- Create denormalized table

CREATE TABLE TotalCategoriesSales (
	  CategoryID int NOT NULL 
		PRIMARY KEY
	, CategoryName nvarchar(15) NOT NULL
	, TotalSales money DEFAULT 0
)
GO

-- Synchronization Procedure
-- Including initial data population

CREATE PROCEDURE SyncTotalCategoriesSales
AS
	TRUNCATE TABLE TotalcategoriesSales

	INSERT TotalCategoriesSales
	SELECT CategoryID
		, CategoryName
		, 0
	FROM Categories

	UPDATE TC
	SET TotalSales =
		(SELECT SUM(OD.UnitPrice * Quantity * (1 - Discount))
		FROM [Order Details] OD
			JOIN Products P
				ON P.ProductID = OD.ProductID
		WHERE P.CategoryID = TC.categoryID)
	FROM TotalCategoriesSales TC
GO

-- Synchronize totals
-- We can run this procedure if necessary

EXEC SyncTotalcategoriesSales

GO


CREATE TRIGGER modOrderDetails
ON [Order Details]
AFTER INSERT, UPDATE, DELETE
AS

	UPDATE TC
	SET TotalSales = TotalSales
		- D.UnitPrice * Quantity * (1 - Discount)
	FROM TotalCategoriesSales TC
		JOIN Products P
			ON P.CategoryID = TC.CategoryID
		JOIN Deleted D
			ON D.ProductID = P.productID

	UPDATE TC
	SET TotalSales = TotalSales
		+ I.UnitPrice * Quantity * (1 - Discount)
	FROM TotalCategoriesSales TC
		JOIN Products P
			ON P.CategoryID = TC.CategoryID
		JOIN Inserted I
			ON I.ProductID = P.productID
GO

-- Select initial values

PRINT CHAR(10) + 'Initial summary values' + CHAR(10)

SELECT *
FROM TotalCategoriesSales

-- Insert a new Order

INSERT Orders (CustomerID)
SELECT 'ALFKI'

DECLARE @id int

SET @id = @@IDENTITY

-- Show the @@identity for later reference

SELECT @id AS OrderID

-- Sell 10 units of Ikura (cat 8) at $30.00 and 10% discount

PRINT CHAR(10) + 'Insert Ikura (cat 8) Order' + CHAR(10)

INSERT [Order Details]
(orderID, ProductID, UnitPrice, Quantity, Discount)
VALUES (@id, 10, 30, 10, 0.1)

-- Sell 20 units of Tofu (cat 7) at $20.00 and 20% discount

PRINT CHAR(10) + 'Insert Tofu (cat 7) Order' + CHAR(10)

INSERT [Order Details]
(orderID, ProductID, UnitPrice, Quantity, Discount)
VALUES (@id, 14, 20, 100, 0.2)

-- Sell 5 units of Queso Cabrales (cat 4) at $20.00 and no discount

PRINT CHAR(10) + 'Insert Queso Cabrales (cat 4) Order' + CHAR(10)

INSERT [Order Details]
(orderID, ProductID, UnitPrice, Quantity, Discount)
VALUES (@id, 11, 20, 5, 0.0)

-- Test the new totals

SELECT *
FROM TotalCategoriesSales

-- Update quantity of the Queso Cabrales to 100

PRINT CHAR(10) + 'Increase Queso Cabrales (cat 4) to 100' + CHAR(10)

UPDATE [Order Details]
SET Quantity = 100
WHERE OrderID = @id
	AND ProductID = 11

-- Test the new totals

SELECT *
FROM TotalCategoriesSales

-- Remove Tofu from this order

PRINT CHAR(10) + 'Remove Tofu (cat 7) Order' + CHAR(10)

DELETE [Order Details]
WHERE OrderID = @id
	AND ProductID = 14

-- Test the new totals

SELECT *
FROM TotalCategoriesSales

-- remove the order completely

PRINT CHAR(10) + 'Remove Ikura (cat 8) Order' + CHAR(10)

DELETE [Order Details]
WHERE OrderID = @id
	AND ProductID = 10

PRINT CHAR(10) + 'Remove Queso Cabrales (cat 4) Order' + CHAR(10)

DELETE [Order Details]
WHERE OrderID = @id
	AND ProductID = 11

DELETE Orders
WHERE OrderID = @id

-- Test the new totals

SELECT *
FROM TotalCategoriesSales
