/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 9.7, Version 1.0, 2001-04-24
****************************************************/

USE Northwind
GO

-- Drop the existing view

IF OBJECT_ID('TotalSalesByCategory') IS NOT NULL
DROP VIEW TotalSalesByCategory
GO

-- Setting required to create an indexed view

SET ANSI_NULLS ON
SET ANSI_PADDING ON
SET ANSI_WARNINGS ON
SET ARITHABORT ON
SET CONCAT_NULL_YIELDS_NULL ON
SET QUOTED_IDENTIFIER ON
SET NUMERIC_ROUNDABORT OFF
GO

-- Create the view
-- With Schemabinding
-- and use two part names for objects

CREATE VIEW TotalSalesByCategory
WITH SCHEMABINDING
AS
	SELECT P.CategoryID
		, C.CategoryName
		, SUM(OD.UnitPrice * Quantity * (1 - Discount)) as Total
		, COUNT_BIG(*) as CB
	FROM dbo.[Order Details] OD
		JOIN dbo.Products P
			ON P.ProductID = OD.ProductID
		JOIN dbo.Categories C
			ON C.CategoryID = P.CategoryID
	GROUP BY P.CategoryID
		, C.categoryName
GO

-- Create the index on the view

CREATE UNIQUE CLUSTERED INDEX ndx_CatTotals
ON TotalSalesByCategory (CategoryName)
GO

-- Use the view to search
-- for 'Confections' totals
-- set SHOWPLAN_TEXT
-- to show that the view is used
-- instead of the base tables

SET SHOWPLAN_TEXT ON
GO

SELECT CategoryID
	, CategoryName
	, Total
FROM TotalSalesByCategory (NOEXPAND)
WHERE CategoryName = 'Confections'
