/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 9.8, Version 1.0, 2001-04-24
****************************************************/

USE Northwind
GO

-- Create a new table

CREATE TABLE NewCategories (
	  CategoryID int NOT NULL
		IDENTITY(1,1) 
		PRIMARY KEY
	, CategoryName nvarchar(15) NOT NULL 
	, Modified smalldatetime
		DEFAULT CURRENT_TIMESTAMP
	, Modifier nchar(128)
		DEFAULT SYSTEM_USER
)
GO

-- Populate the table with data

INSERT NewCategories
(CategoryName)
SELECT CategoryName
FROM Categories
GO

-- Create an AFTER UPDATE trigger

CREATE TRIGGER udt_NewCategories
ON NewCategories
AFTER UPDATE
AS
	UPDATE NewCategories
	SET Modified = CURRENT_TIMESTAMP
		, Modifier = SYSTEM_USER
	FROM NewCategories
		JOIN Inserted
			ON Inserted.CategoryID = NewCategories.CategoryID	
GO

-- Show actual values

SELECT *
FROM NewCategories
WHERE CategoryID = 3
GO

PRINT CHAR(10) + 'Wait for 30 seconds' + CHAR(10)
GO

WAITFOR DELAY '00:00:30'

-- Make sure Guest has permissions to modify
-- and read the NewCategories table

GRANT ALL ON NewCategories TO Guest
GO

-- Impersonate the user Guest

SETUSER 'Guest'
GO

-- Modify Category 3 Name

UPDATE NewCategories
SET CategoryName = 'Confectionery'
WHERE CategoryID = 3

-- Show new values
-- Where the trigger modified automatically
-- the Modified and Modifier columns

SELECT *
FROM NewCategories
WHERE CategoryID = 3
GO

-- Impersonate the previous user (dbo)

SETUSER
GO

-- Drop the demo table
-- it drops automatically the trigger

DROP TABLE NewCategories
