/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 9.12, Version 1.0, 2001-04-24
****************************************************/

USE Northwind
GO

-- Create a new table to store the Budget

CREATE TABLE CategoriesBudget(
	  CategoryID int NOT NULL
		PRIMARY KEY
		REFERENCES Categories (CategoryID)
	, Budget money NOT NULL
		DEFAULT 0
)
GO
-- Add a random budget per category

INSERT CategoriesBudget
SELECT CategoryID
	, ROUND(SIN(CategoryID)* 10000 + 20000, -3)
FROM Categories

GO

-- Create a view to see the name and budget
-- for every Category

CREATE VIEW CategoriesNameBudget
AS
	SELECT C.CategoryID
		, C.CategoryName
		, CB.Budget
	FROM Categories C
		JOIN CategoriesBudget CB
			ON C.categoryID = CB.categoryID

GO

-- Create an INSTEAD OF trigger
-- to UPDATE budgets proportionally

CREATE TRIGGER udtCategoriesNameBudget
ON CategoriesNameBudget
INSTEAD OF UPDATE
AS

	-- Update the Category Name
	
	UPDATE C
	SET C.CategoryName = I.CategoryName
	FROM Inserted I
		JOIN Categories C
			ON C.CategoryID = I.CategoryID
	
	-- Update the Budget
	
	UPDATE CB
	SET CB.Budget = I.Budget
	FROM Inserted I
		JOIN CategoriesBudget CB
			ON CB.CategoryID = I.CategoryID
	
GO

-- Test the view

PRINT CHAR(10) + 'Names and Budgets before the update' + CHAR(10)

SELECT *
FROM CategoriesNameBudget

-- Change name and budget of Category 7

UPDATE CategoriesNameBudget
SET Budget = 35000.0000
	, CategoryName = 'Miscellaneous'
WHERE CategoryID = 7

-- Test the update

PRINT CHAR(10) + 'Names and Budgets after the update' + CHAR(10)

SELECT *
FROM CategoriesNameBudget

-- Back Categories to normal

UPDATE Categories
SET CategoryName = 'Produce'
WHERE CategoryID = 7

DROP VIEW CategoriesNameBudget

DROP TABLE CategoriesName
