/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 9.16, Version 1.0, 2001-04-24
****************************************************/

USE Northwind
GO

-----------------------------
-- Create trigger for insert
-----------------------------

CREATE TRIGGER isrOrderDetails
ON [Order Details]
AFTER INSERT
AS
	IF @@ROWCOUNT = 1
	BEGIN

		-- Single-row operation
		
		UPDATE TC
		SET TotalSales = TotalSales
				+ I.UnitPrice * Quantity * (1 - Discount)
		FROM TotalCategoriesSales TC
			JOIN Products P
				ON P.CategoryID = TC.CategoryID
			JOIN Inserted I
				ON I.ProductID = P.productID
	
	END
	ELSE
	BEGIN
	
		-- Multi-row operation
		
		UPDATE TC
		SET TotalSales = TotalSales
				+ (SELECT SUM(I.UnitPrice * Quantity * (1 - Discount))
					FROM Inserted I
					WHERE I.ProductID = P.productID)
		FROM TotalCategoriesSales TC
			JOIN Products P
				ON P.CategoryID = TC.CategoryID
	
	END
GO

-----------------------------
-- Create trigger for delete
-----------------------------

CREATE TRIGGER delOrderDetails
ON [Order Details]
AFTER DELETE
AS
	IF @@ROWCOUNT = 1
	BEGIN
	
		-- Single-row operation
		
		UPDATE TC
		SET TotalSales = TotalSales
				- D.UnitPrice * Quantity * (1 - Discount)
		FROM TotalCategoriesSales TC
			JOIN Products P
				ON P.CategoryID = TC.CategoryID
			JOIN Deleted D
				ON D.ProductID = P.productID
	
	END
	ELSE
	BEGIN
	
		-- Multi-row operation
		
		UPDATE TC
		SET TotalSales = TotalSales
				- (SELECT SUM(D.UnitPrice * Quantity * (1 - Discount))
					FROM Deleted D
					WHERE D.ProductID = P.productID)
		FROM TotalCategoriesSales TC
			JOIN Products P
				ON P.CategoryID = TC.CategoryID
	
	END
GO

-----------------------------
-- Create trigger for Update
-----------------------------

CREATE TRIGGER udtOrderDetails
ON [Order Details]
AFTER UPDATE
AS
	IF @@ROWCOUNT = 1
	BEGIN
	
		-- Single-row operation
		
		UPDATE TC
		SET TotalSales = TotalSales
				+ I.UnitPrice * I.Quantity * (1 - I.Discount)
		FROM TotalCategoriesSales TC
			JOIN Products P
				ON P.CategoryID = TC.CategoryID
			JOIN Inserted I
				ON I.ProductID = P.productID
		
		UPDATE TC
		SET TotalSales = TotalSales
				- D.UnitPrice * D.Quantity * (1 - D.Discount)
		FROM TotalCategoriesSales TC
			JOIN Products P
				ON P.CategoryID = TC.CategoryID
			JOIN Deleted D
				ON D.ProductID = P.productID
		
	END
	ELSE
	BEGIN
	
		-- Multi-row operation
		
		UPDATE TC
		SET TotalSales = TotalSales
				+ (SELECT SUM(I.UnitPrice * Quantity * (1 - Discount))
					FROM Inserted I
					WHERE I.ProductID = P.productID)
		FROM TotalCategoriesSales TC
			JOIN Products P
				ON P.CategoryID = TC.CategoryID
		
		UPDATE TC
		SET TotalSales = TotalSales
				- (SELECT SUM(D.UnitPrice * Quantity * (1 - Discount))
					FROM Deleted D
					WHERE D.ProductID = P.productID)
		FROM TotalCategoriesSales TC
			JOIN Products P
				ON P.CategoryID = TC.CategoryID
		
	END
GO
