/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 9.20, Version 1.0, 2001-04-25
****************************************************/

USE Northwind
GO

-- Add the column SaleTotal to the
-- Orders table

ALTER TABLE Orders
ADD SaleTotal money NULL

-- Add the column SaleTotal to the
-- Employees table

ALTER TABLE Employees
ADD SaleTotal money NULL

-- Add the column SaleTotal to the
-- Customers table

ALTER TABLE Customers
ADD SaleTotal money NULL

GO

-- Initialize the data

UPDATE Orders
SET SaleTotal =(
	SELECT SUM([Order Details].UnitPrice * Quantity * (1 - Discount))
	FROM [Order Details]
	WHERE [Order Details].OrderID = Orders.OrderID
	)

UPDATE Employees
SET SaleTotal =(
	SELECT SUM(Orders.SaleTotal)
	FROM Orders
	WHERE Orders.EmployeeID = Employees.EmployeeID
	)

UPDATE Customers
SET SaleTotal =(
	SELECT SUM(Orders.SaleTotal)
	FROM Orders
	WHERE Orders.CustomerID = Customers.CustomerID
	)

GO

-- Create nested triggers

CREATE TRIGGER isrTotalOrderDetails
ON [Order details]
AFTER INSERT, DELETE, UPDATE
AS

	IF @@rowcount = 1
	
		-- Single-row operation
		
		UPDATE Orders
		SET SaleTotal = SaleTotal
			+ ISNULL(
				(SELECT UnitPrice * Quantity * (1 - Discount)
				FROM Inserted
				WHERE Inserted.OrderID = Orders.OrderID), 0)
			- ISNULL(
				(SELECT UnitPrice * Quantity * (1 - Discount)
				FROM Deleted
				WHERE Deleted.OrderID = Orders.OrderID), 0)
	
	ELSE
	
		-- Multi-row operation
		
		UPDATE Orders
		SET SaleTotal = SaleTotal
			+ ISNULL(
				(SELECT SUM(UnitPrice * Quantity * (1 - Discount))
				FROM Inserted
				WHERE Inserted.OrderID = Orders.OrderID), 0)
			- ISNULL(
				(SELECT SUM(UnitPrice * Quantity * (1 - Discount))
				FROM Deleted
				WHERE Deleted.OrderID = Orders.OrderID), 0)
	
GO


CREATE TRIGGER isrTotalOrders
ON Orders
AFTER INSERT, DELETE, UPDATE
AS

	IF @@rowcount = 1
	BEGIN
		-- Single-row operation
		
		UPDATE Employees
		SET SaleTotal = SaleTotal
			+ ISNULL(
				(SELECT SaleTotal
				FROM Inserted
				WHERE Inserted.EmployeeID = Employees.EmployeeID), 0)
			- ISNULL(
				(SELECT SaleTotal
				FROM Deleted
				WHERE Deleted.EmployeeID = Employees.EmployeeID), 0)
		
		UPDATE Customers
		SET SaleTotal = SaleTotal
			+ ISNULL(
				(SELECT SaleTotal
				FROM Inserted
				WHERE Inserted.CustomerID = Customers.CustomerID), 0)
		- ISNULL(
				(SELECT SaleTotal
				FROM Deleted
				WHERE Deleted.CustomerID = Customers.CustomerID), 0)
		
	END
	ELSE
	BEGIN
	
		-- Multi-row operation
		
		UPDATE Employees
		SET SaleTotal = SaleTotal
			+ ISNULL(
				(SELECT SUM(SaleTotal)
				FROM Inserted
				WHERE Inserted.EmployeeID = Employees.EmployeeID), 0)
			- ISNULL(
				(SELECT SUM(SaleTotal)
				FROM Deleted
				WHERE Deleted.EmployeeID = Employees.EmployeeID), 0)
		
		UPDATE Customers
		SET SaleTotal = SaleTotal
			+ ISNULL(
				(SELECT SUM(SaleTotal)
				FROM Inserted
				WHERE Inserted.CustomerID = Customers.CustomerID), 0)
			- ISNULL(
				(SELECT SUM(SaleTotal)
				FROM Deleted
				WHERE Deleted.CustomerID = Customers.CustomerID), 0)
	
	END
GO

-- Updating Order Details
-- and forcing the nested triggers
-- execution

UPDATE [Order Details]
SET quantity = 100
WHERE orderid = 10248
	AND productid = 11

-- Testing totals in Orders table

SELECT CustomerID
	, EmployeeID
	, SaleTotal 
FROM orders
WHERE OrderID =  10248

SELECT SUM([Order Details].UnitPrice * Quantity * (1 - Discount))
FROM [Order Details]
WHERE OrderID =  10248

-- Testing totals in Employees

SELECT SaleTotal
FROM Employees
WHERE EmployeeID = 5

SELECT SUM(SaleTotal)
FROM Orders
WHERE EmployeeID =  5

-- Testing totals in Customers

SELECT SaleTotal
FROM Customers
WHERE CustomerID = 'VINET'

SELECT SUM(SaleTotal)
FROM Orders
WHERE CustomerID =  'VINET'
GO

-- Dropping triggers

DROP TRIGGER isrTotalOrderDetails

DROP TRIGGER isrTotalOrders
