/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 9.21, Version 1.0, 2001-04-25
****************************************************/

USE Northwind
GO

CREATE TRIGGER tr_OrderDetails_TotalOrders
ON [Order details]
AFTER INSERT, DELETE, UPDATE
AS

	IF @@rowcount = 1
	
		-- Single-row operation
		
		UPDATE Orders
		SET SaleTotal = SaleTotal
			+ ISNULL(
				(SELECT UnitPrice * Quantity * (1 - Discount)
				FROM Inserted
				WHERE Inserted.OrderID = Orders.OrderID), 0)
			- ISNULL(
				(SELECT UnitPrice * Quantity * (1 - Discount)
				FROM Deleted
				WHERE Deleted.OrderID = Orders.OrderID), 0)
	
	ELSE
	
		-- Multi-row operation
		
		UPDATE Orders
		SET SaleTotal = SaleTotal
			+ ISNULL(
				(SELECT SUM(UnitPrice * Quantity * (1 - Discount))
				FROM Inserted
				WHERE Inserted.OrderID = Orders.OrderID), 0)
			- ISNULL(
				(SELECT SUM(UnitPrice * Quantity * (1 - Discount))
				FROM Deleted
				WHERE Deleted.OrderID = Orders.OrderID), 0)
	
GO


CREATE TRIGGER tr_OrderDetails_TotalEmployees
ON [Order details]
AFTER INSERT, DELETE, UPDATE
AS

	IF @@rowcount = 1
	
		-- Single-row operation
		
		UPDATE Employees
		SET SaleTotal = SaleTotal
			+ ISNULL(
				(SELECT UnitPrice * Quantity * (1 - Discount)
				FROM Inserted
					JOIN Orders
						ON Inserted.OrderID = Orders.OrderID
				WHERE Orders.EmployeeID = Employees.EmployeeID), 0)
			- ISNULL(
				(SELECT UnitPrice * Quantity * (1 - Discount)
				FROM Deleted
					JOIN Orders
						ON Deleted.OrderID = Orders.OrderID
				WHERE Orders.EmployeeID = Employees.EmployeeID), 0)
	
	ELSE
	
		-- Multi-row operation
		
		UPDATE Employees
		SET SaleTotal = SaleTotal
			+ ISNULL(
				(SELECT SUM(UnitPrice * Quantity * (1 - Discount))
				FROM Inserted
					JOIN Orders
						ON Inserted.OrderID = Orders.OrderID
				WHERE Orders.EmployeeID = Employees.EmployeeID), 0)
			- ISNULL(
				(SELECT SUM(UnitPrice * Quantity * (1 - Discount))
				FROM Deleted
					JOIN Orders
						ON Deleted.OrderID = Orders.OrderID
				WHERE Orders.EmployeeID = Employees.EmployeeID), 0)

GO

CREATE TRIGGER tr_OrderDetails_TotalCustomers
ON [Order details]
AFTER INSERT, DELETE, UPDATE
AS

	IF @@rowcount = 1
	
		-- Single-row operation
		
		UPDATE Customers
		SET SaleTotal = SaleTotal
			+ ISNULL(
				(SELECT UnitPrice * Quantity * (1 - Discount)
				FROM Inserted
					JOIN Orders
						ON Inserted.OrderID = Orders.OrderID
				WHERE Orders.CustomerID = Customers.CustomerID), 0)
			- ISNULL(
				(SELECT UnitPrice * Quantity * (1 - Discount)
				FROM Deleted
					JOIN Orders
						ON Deleted.OrderID = Orders.OrderID
				WHERE Orders.CustomerID = Customers.CustomerID), 0)
		
	ELSE
	
		-- Multi-row operation
		
		UPDATE Customers
		SET SaleTotal = SaleTotal
			+ ISNULL(
				(SELECT SUM(UnitPrice * Quantity * (1 - Discount))
				FROM Inserted
					JOIN Orders
						ON Inserted.OrderID = Orders.OrderID
				WHERE Orders.CustomerID = Customers.CustomerID), 0)
			- ISNULL(
				(SELECT SUM(UnitPrice * Quantity * (1 - Discount))
				FROM Deleted
					JOIN Orders
						ON Deleted.OrderID = Orders.OrderID
				WHERE Orders.CustomerID = Customers.CustomerID), 0)
GO

DROP TRIGGER tr_OrderDetails_TotalOrders

DROP TRIGGER tr_OrderDetails_TotalCustomers

DROP TRIGGER tr_OrderDetails_TotalEmployees

GO
