/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 10.11, Version 1.0, 2001-04-25
****************************************************/

USE Northwind
GO

-- dbo.MaxProductID

SELECT dbo.MaxProductID() AS MaxProductID
GO

-- dbo.WhoWhere

SELECT dbo.WhoWhere() AS [Who from Where]
GO

-- Create a table using WHoWhere
-- As a DEFAULT constraint

IF OBJECT_ID('TestWhoWhere') IS NOT NULL
	DROP TABLE TestWhoWhere
GO

CREATE TABLE TestWhoWhere(
	  ID int IDENTITY (1,1)
		PRIMARY KEY
	, Name nvarchar(40)
	, WhoWhere nvarchar(256) 
		DEFAULT dbo.WhoWhere()
)

INSERT TestWhoWhere (Name)
VALUES ('New record')

SELECT *
FROM TestWhoWhere
GO

-- Create a trigger to Update automatically
-- Who did it and from where the change was done

CREATE TRIGGER trWhoWhere
ON TestWhoWhere
AFTER INSERT, UPDATE
AS
    UPDATE TestWhoWhere
    SET WhoWhere = dbo.WhoWhere()
    FROM TestWhoWhere T
    	JOIN Inserted I
    		ON T.ID = I.ID
GO

INSERT TestWhoWhere
(Name, WhoWhere)
VALUES
('More records', 'nobody from nowhere')

SELECT *
FROM TestWhoWhere

-- dbo.Today

SELECT dbo.Today() AS Today

-- Inserting an Order in the present month

INSERT Orders (CustomerID, OrderDate, ShippedDate)
VALUES ('WELLI', '2000-11-23', '2000-12-12')

-- Searching for Orders in the last 30 days

SELECT OrderID, CustomerID, OrderDate, ShippedDate
FROM Orders
WHERE DATEDIFF(day, ShippedDate, dbo.Today()) < 30
GO

-- dbo.PRand

SELECT dbo.PRand() AS PRand
GO

-- dbo.Get3Rand

SELECT dbo.Get3Rand() as Get3Rand1
GO

SELECT dbo.Get3Rand() as Get3Rand2
GO

-- dbo.NewPRand

SELECT dbo.NewPRand() as NewPRand
GO

-- dbo.MediumProductUnitPrice

SELECT dbo.MediumProductUnitPrice()
GO

-- Get the Medium 10 products
-- by UnitPrice

PRINT CHAR(10) + 'Medium 10 Products' + CHAR(10)

SELECT *
FROM (
	SELECT TOP 6
		UnitPrice, ProductID, ProductName
	FROM Products
	WHERE UnitPrice >= dbo.MediumProductUnitPrice()
	ORDER BY UnitPrice ASC
	) AS A

UNION

SELECT *
FROM (
	SELECT TOP 5
		UnitPrice, ProductID, ProductName
	FROM Products
	WHERE UnitPrice <= dbo.MediumProductUnitPrice()
	ORDER BY UnitPrice DESC
	) AS B

ORDER BY UnitPrice ASC

-- dbo.fn_FV

SELECT 0.07 AS Rate
	, 36 AS NPer
	, 1000 AS Pmt
	, 10000 AS Pv
	, 0 AS Type
	, dbo.fn_FV(0.07, 36, 1000, 10000, 0) AS FV

-- dbo.SimpleEncrypt

SELECT dbo.SimpleEncrypt('Hello World')
	AS [Encrypted Message]

-- dbo.SimpleDecrypt

SELECT dbo.SimpleDecrypt('Hello World')
	AS [Decrypted version of a non-encrypted message]

SELECT dbo.SimpleDecrypt(dbo.SimpleEncrypt('Hello World'))
	AS [Decrypted version of an encrypted message]

-- dbo.ANG_HMS

SELECT dbo.ANG_HMS(12.3456, 3, 'g') 
	AS 'GRAD to HMS Using ANG_HMS'

SELECT dbo.ANG_HMS(12.3456, 3, CHAR(176)) 
	AS 'DEG to HMS Using ANG_HMS'

-- dbo.GRAD_HMS

SELECT dbo.GRAD_HMS(12.3456, DEFAULT) 
	AS 'GRAD to HMS Using GRAD_HMS'

-- dbo.DEG_HMS

SELECT dbo.DEG_HMS(12.3456, 2) 
	AS 'DEG to HMS Using DEG_HMS'

-- dbo.HMS_ANG

SELECT dbo.HMS_ANG('12g 34m 56s789', 'g') 
	AS 'HMS to GRAD Using HMS_ANG'

SELECT dbo.HMS_ANG('12 34m 56s789', '') 
	AS 'HMS to DEG Using HMS_ANG'

-- dbo.HMS_DEG

SELECT dbo.HMS_DEG('12 34m 56s789') 
	AS 'HMS to DEG Using HMS_DEG'

-- dbo.HMS_GRAD

SELECT dbo.HMS_GRAD('12g 34m 56s789') 
	AS 'HMS to GRAD Using HMS_GRAD'

SELECT dbo.GRAD_HMS(dbo.HMS_GRAD('12g 34m 56s789'), 3)
	AS 'HMS to GRAD and to HMS again using HMS_GRAD and GRAD_HMS'

GO
