/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 10.14, Version 1.0, 2001-04-25
****************************************************/

USE Northwind
GO

-- Returns customers from a specific country

CREATE FUNCTION dbo.GetCustomersFromCountry
(@country nvarchar(15))
RETURNS TABLE
AS
	RETURN (
		SELECT *
		FROM Customers
		WHERE Country = @country
	)
GO

-- Returns USA-based customers

CREATE FUNCTION dbo.GetCustomersFromUSA
()
RETURNS TABLE
AS
	RETURN (
		SELECT *
		FROM dbo.GetCustomersFromCountry(N'USA')
	)
GO

-- Returns the orders from a specific day

CREATE FUNCTION dbo.GetOrdersFromDay
(@date as smalldatetime)
RETURNS TABLE
AS
	RETURN (
		SELECT *
		FROM Orders
		WHERE DATEDIFF(day, OrderDate, @date) = 0
	)
GO

-- Returns orders from today

CREATE FUNCTION dbo.GetOrdersFromToday
()
RETURNS TABLE
AS
	RETURN (
		SELECT *
		FROM Orders
		WHERE DATEDIFF(day, OrderDate, dbo.Today()) = 0
	)
GO

-- Returns Orders with the total order value

CREATE FUNCTION dbo.OrdersWithValue
()
RETURNS TABLE
AS
	RETURN (
		SELECT O.*, TotalValue
		FROM Orders O
			JOIN (
				SELECT OrderID
					, SUM(dbo.TotalPrice(Quantity, UnitPrice, Discount))
						AS TotalValue
				FROM [Order Details]
				GROUP BY OrderID
			) AS OD
				ON O.OrderID = OD.OrderID
	)
GO

-- Returns orders with a value greater than
-- a specific target value

CREATE FUNCTION dbo.OrdersByValue
(@total money)
RETURNS TABLE
AS
	RETURN (
		SELECT *
		FROM dbo.OrdersWithValue()
		WHERE TotalValue > @total
	)
GO

-- Returns the top 10 orders by total value

CREATE FUNCTION dbo.TopTenOrders
()
RETURNS TABLE
AS
	RETURN (
		SELECT TOP 10 WITH TIES *
		FROM dbo.OrdersWithValue()
		ORDER BY TotalValue DESC
	)

GO
