/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation,
** Source code. Listing 11.4, Version 1.0, 2001-04-04
****************************************************/


USE Northwind
GO

SET NOCOUNT ON
GO

-- In this case we combine the values returned by two subqueries 
-- to get the medium unit price

SELECT (
		(
			SELECT MIN(Unitprice)
			FROM Products
		) +
		(
			SELECT MAX(Unitprice)
			FROM Products
		)
	)/2 as NewPrice

-- This query is not practically useful, 
-- but it shows more choices on designing subqueries

SELECT 1, 2
	, (
		SELECT 3
	)
GO

-- This query uses two subqueries to retrieve one single row 
-- with the Maximum and Average UnitPrice

SELECT (
		SELECT AVG(Unitprice)
		FROM Products
	) as AvgPrice
	, (
		SELECT MAX(Unitprice)
		FROM Products
	) as MaxPrice
GO

-- Compare the UnitPrice of every product
-- with the Average UnitPrice, produced by a subquery

SELECT ProductName, UnitPrice
	, (
		SELECT AVG(Unitprice)
		FROM Products
	) as AvgPrice
FROM Products
WHERE CategoryID = 2
GO

-- Updates the UnitPrice of the product 11 to 
-- 20% more than the maximum UnitPrice.


UPDATE Products
SET UnitPrice = (
		SELECT MAX(Unitprice)
		FROM Northwind..Products
	) * 1.2
WHERE ProductID = 11

-- Show the product with maximum UnitPrice

SELECT ProductName, UnitPrice
FROM Products P
WHERE Unitprice = (
		SELECT Max(UnitPrice) MPrice
		FROM Products
	)

-- You want to retrieve the Categories with average Unitprice 
-- greater than the overall products average price

SELECT CategoryID, AVG(UnitPrice) AS 'Average Price'
FROM Products P
GROUP BY CategoryID
HAVING AVG(UnitPrice) > (
		SELECT AVG(UnitPrice) MPrice
		FROM Products
	)
