/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation,
** Source code. Listing 12.22, Version 1.0, 2001-04-04
****************************************************/


USE Northwind
GO

SET NOCOUNT ON
GO

-- Declare some cursors

DECLARE CCategories CURSOR LOCAL DYNAMIC
FOR
	SELECT CategoryName
	FROM Categories

DECLARE CCustomers CURSOR LOCAL FAST_FORWARD
FOR
	SELECT CompanyName
	FROM Customers

DECLARE COrdersComplete CURSOR GLOBAL KEYSET
FOR
	SELECT O.OrderID
		, OrderDate
		, C.CustomerID
		, CompanyName
		, P.ProductID
		, ProductName
		, Quantity
		, OD.UnitPrice
		, Discount
	FROM Orders O
		JOIN [Order Details] OD
			ON OD.OrderID = O.OrderID
		JOIN Customers C
			ON C.CustomerID = O.CustomerID
		JOIN Products P
			ON P.ProductID = OD.ProductID

-- Declare a cursor variable to hold
-- results from the stored procedures

DECLARE @OutputCursor AS CURSOR

-- Get information about declared local cursors

EXEC sp_cursor_list @OutputCursor OUTPUT, 1

-- deallocate the cursor, so we can reuse the cursor variable

DEALLOCATE @OutputCursor

-- Or get information about declared global cursors

EXEC sp_cursor_list @OutputCursor OUTPUT, 2

-- deallocate the cursor, so we can reuse the cursor variable

DEALLOCATE @OutputCursor

-- Or get information about declared global and local cursors
-- note that status = -1 means cursor closed

PRINT CHAR(10) 
	+ 'sp_cursor_list cursor OUTPUT' 
	+ CHAR(10)

EXEC sp_cursor_list @OutputCursor OUTPUT, 3

FETCH NEXT FROM @OutputCursor

WHILE @@FETCH_STATUS = 0
	FETCH NEXT FROM @OutputCursor

-- deallocate the cursor, so we can reuse the cursor variable

DEALLOCATE @OutputCursor

-- Open the CCategories cursor

OPEN CCategories

-- Get information about a cursor
-- note that status = 1 means cursor open

EXEC sp_describe_cursor @OutputCursor OUTPUT
	, N'local'
	, N'CCategories'

PRINT CHAR(10) 
	+ 'sp_describe_cursor cursor OUTPUT' 
	+ CHAR(10)

FETCH NEXT FROM @OutputCursor

WHILE @@FETCH_STATUS = 0
	FETCH NEXT FROM @OutputCursor

-- deallocate the cursor, so we can reuse the cursor variable

DEALLOCATE @OutputCursor

CLOSE CCategories

-- Open the CCustomers cursor

OPEN CCustomers

-- Get information about a cursor
-- note that status = 1 means cursor open

EXEC sp_describe_cursor_columns @OutputCursor OUTPUT
	, N'local'
	, N'CCustomers'

PRINT CHAR(10) 
	+ 'sp_describe_cursor_columns cursor OUTPUT' 
	+ CHAR(10)

FETCH NEXT FROM @OutputCursor

WHILE @@FETCH_STATUS = 0
	FETCH NEXT FROM @OutputCursor

-- deallocate the cursor, so we can reuse the cursor variable

DEALLOCATE @OutputCursor

CLOSE CCustomers

-- Open the CCategories cursor

OPEN COrdersComplete

-- Get information about a cursor
-- note that status = 1 means cursor open

EXEC sp_describe_cursor_tables @OutputCursor OUTPUT
	, N'global'
	, N'COrdersComplete'

PRINT CHAR(10) 
	+ 'sp_dscribe_cursor_tables cursor OUTPUT' 
	+ CHAR(10)

FETCH NEXT FROM @OutputCursor

WHILE @@FETCH_STATUS = 0
	FETCH NEXT FROM @OutputCursor

DEALLOCATE @OutputCursor

CLOSE COrdersComplete

DEALLOCATE CCategories

DEALLOCATE CCustomers

DEALLOCATE COrdersComplete
