/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation,
** Source code. Listing 13.9, Version 1.0, 2001-04-04
****************************************************/


USE Northwind
GO

------------------------
-- Without Error Control
------------------------

DECLARE @PID int, @OID int

PRINT CHAR(10) 
	+ 'Start a transaction without error control' 
	+ CHAR(10)

BEGIN TRAN

	INSERT Products (ProductName, CategoryID, UnitPrice)
	VALUES ('New Cars Magazine Year Subscription', 10, 35.0)
	
	SET @PID = SCOPE_IDENTITY()
	
	INSERT Orders (CustomerID, OrderDate)
	VALUES ('COMMI', '2000-11-23')
	
	SET @OID = SCOPE_IDENTITY()
	
	INSERT [Order Details]
		(OrderID, ProductID, UnitPrice, Quantity, Discount)
	SELECT @OID, ProductID, UnitPrice, 1, 0.3
	FROM Products
	WHERE ProductID = @PID
	
COMMIT TRAN
	
PRINT CHAR(10)
+ 'The transaction has been partially applied'
+ CHAR(10)
	
SELECT ProductName
FROM Products
WHERE ProductID = @PID

SELECT CustomerID, OrderDate
FROM Orders
WHERE OrderID = @OID

SELECT UnitPrice, Quantity
FROM [Order Details]
WHERE ProductID = @PID
GO
	
---------------------
-- With Error Control
---------------------

DECLARE @PID int, @OID int

PRINT CHAR(10)
	+ 'Start a transaction with error control'
	+ CHAR(10)

BEGIN TRAN
	
	INSERT Products (ProductName, CategoryID, UnitPrice)
	VALUES ('New Cars Magazine Year Subscription', 10, 35.0)
	
	IF @@ERROR <> 0
		GOTO CancelOrder
	
	SET @PID = SCOPE_IDENTITY()
	
	INSERT Orders (CustomerID, OrderDate)
	VALUES ('COMMI', '2000-11-23')
	
	IF @@ERROR <> 0
		GOTO CancelOrder
	
	SET @OID = SCOPE_IDENTITY()
	
	INSERT [Order Details]
		(OrderID, ProductID, UnitPrice, Quantity, Discount)
	SELECT @OID, ProductID, UnitPrice, 1, 0.3
	FROM Products
	WHERE ProductID = @PID
	
	IF @@ERROR <> 0 OR @@ROWCOUNT=0
		GOTO CancelOrder

COMMIT TRAN
GOTO CheckOrder

CancelOrder:

	ROLLBACK TRAN

CheckOrder:

	PRINT CHAR(10)
		+ 'The transaction has been completely rolled back'
		+ CHAR(10)
	
	SELECT ProductName
	FROM Products
	WHERE ProductID = @PID
	
	SELECT CustomerID, OrderDate
	FROM Orders
	WHERE OrderID = @OID
	
	SELECT UnitPrice, Quantity
	FROM [Order Details]
	WHERE ProductID = @PID
