/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation,
** Source code. Listing 14.15, Version 1.0, 2001-04-04
****************************************************/


USE Northwind
GO

---------------------------
-- CreaWavFmt
---------------------------
CREATE PROCEDURE CreaWavFmt
	  @dir varchar(255) -- directory ended with '\'
	, @length int -- file length
AS
	/*
	** This is the required step to import
	** image files with BULK INSERT
	**
	** We should do it manually, but we
	** have xp_cmdshell for?
	*/
	
	DECLARE @cmd varchar(8000)
	
	-- Remove wav.fmt file if exists
	
	SET @cmd = 'del '
		+ @dir + 'wav.fmt'
	
	EXEC master.dbo.xp_cmdshell @cmd, no_output
	
	-- Create the first line of the format file
	
	SET @cmd = 'echo 8.0 >>'
		+ @dir + 'wav.fmt'
	
	EXEC master.dbo.xp_cmdshell @cmd, no_output
	
	-- Write the second line to the file
	
	SET @cmd = 'echo 1 >>'
		+ @dir + 'wav.fmt'
	
	EXEC master.dbo.xp_cmdshell @cmd, no_output
	
	/*
	** Add the third line to the file, specifying:
	** 1 (the first field = entire file)
	** SQLIMAGE as datatype
	** 0 as field prefix length
	** length of the field (file in this case)
	** no field separator
	** third field on the table
	** Wav field
	** Empty collation
	*/
	
	SET @cmd = 'echo 1 SQLIMAGE 0 '
		+ CONVERT(varchar(10), @length)
		+ ' "" 3 wav "" >>'
		+ @dir + 'wav.fmt'
	
	EXEC master.dbo.xp_cmdshell @cmd, no_output
	
	-- wav.fmt is created already for this file

GO

---------------------------
-- InsertWavFiles
---------------------------

CREATE PROCEDURE InsertWavFiles
	@dir varchar(255)
AS

	DECLARE @sdir varchar(256)
	
	/*
	** Create temporary table to hold
	** directory contents
	*/
	
	CREATE TABLE #tdir(
		  FileDir varchar(200) NULL
		, length int NULL)
	
	SET @sdir = 'dir '
		+ @dir + '*.WAV'
	
	INSERT #tdir (FileDir)
	EXEC master.dbo.xp_cmdshell @sdir
	
	-- Filter undesired rows
	-- you can add your own conditions
	
	DELETE #tdir
	WHERE FileDir NOT LIKE '%.WAV'
		OR FileDir IS NULL
	
	-- Obtain file length and
	-- filename cleansing
	--
	-- You could check with
	-- EXEC master.dbo.xp_cmdshell 'dir c:\*.*'
	-- that lengths are correct
	
	UPDATE #tdir
	SET length = CONVERT(int, 
						CONVERT(money, 
							LTRIM(
								RTRIM(
									SUBSTRING(FileDir, 20, 20)
									)
							), 1)
						)
		, FileDir = LTRIM(RTRIM(SUBSTRING(FileDir, 40, 40)))
	
	DECLARE @file varchar(256)
	DECLARE @length int
	DECLARE @sql varchar(8000)
	
	DECLARE c_files CURSOR
	FOR 
		SELECT FileDir, length
		FROM #tdir
	
	OPEN c_files
	
	FETCH NEXT FROM c_files INTO @file, @length
	
	WHILE @@FETCH_STATUS = 0
	BEGIN
	
		-- Create bcp.fmt file to import the file
		
		EXEC CreaWavFmt @dir, @length
		
		-- Import the file
		
		SET @sql ='BULK INSERT WAVFiles FROM '''
			+ @dir
			+ @file
			+ ''' WITH (FORMATFILE = '''
			+ @dir
			+ 'wav.fmt'')'
		
		EXECUTE (@sql)
		
		-- Update the imported record
		
		UPDATE WAVFiles
		SET FUllFileName = @dir + @file
		WHERE ID = IDENT_CURRENT('WAVFiles')
		
		FETCH NEXT FROM c_files INTO @file, @length
	END
	
	CLOSE c_files
	
	DEALLOCATE c_files
	
	DROP TABLE #tdir
	
GO

-------------------------------------
-- Test the InsertWavFiles procedure
-------------------------------------

EXEC InsertWavFiles 'd:\winnt\media\'

SELECT ID
	, DATALENGTH(wav) AS WAVELength
	, FullFileName
FROM WAVFiles
