/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation,
** Source code. Listing 15.22, Version 1.0, 2001-04-04
****************************************************/


USE Northwind
GO

-- Create the partitioned table
-- RegionID is the partitioning column
-- it is part of the PRIMARY KEY
-- and it has a check constraint
-- to delimit ranges per table

-- RegionID = 3 North

CREATE TABLE SalesInfoNorth (
	  OrderID int NOT NULL
	, RegionID int NOT NULL
		CHECK (RegionID = 3)
	, SaleDate datetime
	, Amount money
	, EmployeeID int
	, CONSTRAINT PK_SI_North
		PRIMARY KEY (OrderID, RegionID)
)


-- RegionID = 4 South

CREATE TABLE SalesInfoSouth (
	  OrderID int NOT NULL
	, RegionID int NOT NULL
		CHECK (RegionID = 4)
	, SaleDate datetime
	, Amount money
	, EmployeeID int
	, CONSTRAINT PK_SI_South
		PRIMARY KEY (OrderID, RegionID)
)

-- RegionID = 1 East

CREATE TABLE SalesInfoEast (
	  OrderID int NOT NULL
	, RegionID int NOT NULL
		CHECK (RegionID = 1)
	, SaleDate datetime
	, Amount money
	, EmployeeID int
	, CONSTRAINT PK_SI_East
		PRIMARY KEY (OrderID, RegionID)
)

-- RegionID = 2 West

CREATE TABLE SalesInfoWest (
	  OrderID int NOT NULL
	, RegionID int NOT NULL
		CHECK (RegionID = 2)
	, SaleDate datetime
	, Amount money
	, EmployeeID int
	, CONSTRAINT PK_SI_West
		PRIMARY KEY (OrderID, RegionID)
)
GO

-- Create a View that gets the entire
-- SalesInfo informations
-- Note the use of UNION ALL
-- This is the Partitioned View

CREATE VIEW SalesInfo
AS
		SELECT *
		FROM SalesInfoNorth
	UNION ALL
		SELECT *
		FROM SalesInfoSouth
	UNION ALL
		SELECT *
		FROM SalesInfoEast
	UNION ALL
		SELECT *
		FROM SalesInfoWest
GO

-- Populate the partitioned tables
-- using the SalesInfo view directly
-- The partitioned view mechanism
-- will send every row to the appropriate
-- destination table automatically.

INSERT SalesInfo
	SELECT o.OrderID
		, T.RegionID
		, O.OrderDate
		, sum(UnitPrice * Quantity * (1-Discount))
		, O.EmployeeID
	FROM Orders O
		JOIN [Order Details] OD
			ON O.OrderID = OD.OrderID
		JOIN EmployeeTerritories ET
			ON ET.EmployeeID = O.EmployeeID
		JOIN Territories T
			ON T.TerritoryID = ET.TerritoryID
	GROUP BY O.OrderID, T.RegionID, O.OrderDate, O.EmployeeID
GO

-- Checking number of rows in every table and total

SELECT COUNT(*) AS CountNorth
FROM SalesInfoNorth

SELECT COUNT(*) AS CountSouth
FROM SalesInfoSouth

SELECT COUNT(*) AS CountEast
FROM SalesInfoEast

SELECT COUNT(*) AS CountWest
FROM SalesInfoWest

SELECT COUNT(*) AS CountTotal
FROM SalesInfo
GO
